# -*- coding: utf-8 -*-
try:
    from django.template import Lexer, TOKEN_TEXT, TOKEN_VAR, TOKEN_BLOCK
except ImportError:
    # Django 1.8 moved most stuff to .base
    from django.template.base import Lexer, TOKEN_TEXT, TOKEN_VAR, TOKEN_BLOCK

from django.utils.translation.trans_real import (
    inline_re, block_re, endblock_re, plural_re, constant_re)
from django.utils.encoding import smart_text


def extract_django(fileobj, keywords, comment_tags, options):
    """Extract messages from Django template files.

    :param fileobj: the file-like object the messages should be extracted from
    :param keywords: a list of keywords (i.e. function names) that should
                     be recognized as translation functions
    :param comment_tags: a list of translator tags to search for and
                         include in the results
    :param options: a dictionary of additional options (optional)
    :return: an iterator over ``(lineno, funcname, message, comments)``
             tuples
    :rtype: ``iterator``
    """
    intrans = False
    inplural = False
    message_context = None
    singular = []
    plural = []
    lineno = 1

    encoding = options.get('encoding', 'utf8')
    text = fileobj.read().decode(encoding)

    try:
        text_lexer = Lexer(text)
    except TypeError:
        # Django 1.9 changed the way we invoke Lexer; older versions
        # require two parameters.
        text_lexer = Lexer(text, None)

    for t in text_lexer.tokenize():
        lineno += t.contents.count('\n')
        if intrans:
            if t.token_type == TOKEN_BLOCK:
                endbmatch = endblock_re.match(t.contents)
                pluralmatch = plural_re.match(t.contents)
                if endbmatch:
                    if inplural:
                        if message_context:
                            yield (
                                lineno,
                                'npgettext',
                                [smart_text(message_context),
                                 smart_text(u''.join(singular)),
                                 smart_text(u''.join(plural))],
                                [],
                            )
                        else:
                            yield (
                                lineno,
                                'ngettext',
                                (smart_text(u''.join(singular)),
                                 smart_text(u''.join(plural))),
                                [])
                    else:
                        if message_context:
                            yield (
                                lineno,
                                'pgettext',
                                [smart_text(message_context),
                                 smart_text(u''.join(singular))],
                                [],
                            )
                        else:
                            yield (
                                lineno,
                                None,
                                smart_text(u''.join(singular)),
                                [])

                    intrans = False
                    inplural = False
                    message_context = None
                    singular = []
                    plural = []
                elif pluralmatch:
                    inplural = True
                else:
                    raise SyntaxError('Translation blocks must not include '
                                      'other block tags: %s' % t.contents)
            elif t.token_type == TOKEN_VAR:
                if inplural:
                    plural.append('%%(%s)s' % t.contents)
                else:
                    singular.append('%%(%s)s' % t.contents)
            elif t.token_type == TOKEN_TEXT:
                if inplural:
                    plural.append(t.contents)
                else:
                    singular.append(t.contents)
        else:
            if t.token_type == TOKEN_BLOCK:
                imatch = inline_re.match(t.contents)
                bmatch = block_re.match(t.contents)
                cmatches = constant_re.findall(t.contents)
                if imatch:
                    g = imatch.group(1)
                    if g[0] == '"':
                        g = g.strip('"')
                    elif g[0] == "'":
                        g = g.strip("'")
                    message_context = imatch.group(3)
                    if message_context:
                        # strip quotes
                        message_context = message_context[1:-1]
                        yield (
                            lineno,
                            'pgettext',
                            [smart_text(message_context), smart_text(g)],
                            [],
                        )
                        message_context = None
                    else:
                        yield lineno, None, smart_text(g), []
                elif bmatch:
                    if bmatch.group(2):
                        message_context = bmatch.group(2)[1:-1]
                    for fmatch in constant_re.findall(t.contents):
                        yield lineno, None, smart_text(fmatch), []
                    intrans = True
                    inplural = False
                    singular = []
                    plural = []
                elif cmatches:
                    for cmatch in cmatches:
                        yield lineno, None, smart_text(cmatch), []
            elif t.token_type == TOKEN_VAR:
                parts = t.contents.split('|')
                cmatch = constant_re.match(parts[0])
                if cmatch:
                    yield lineno, None, smart_text(cmatch.group(1)), []
                for p in parts[1:]:
                    if p.find(':_(') >= 0:
                        p1 = p.split(':', 1)[1]
                        if p1[0] == '_':
                            p1 = p1[1:]
                        if p1[0] == '(':
                            p1 = p1.strip('()')
                        if p1[0] == "'":
                            p1 = p1.strip("'")
                        elif p1[0] == '"':
                            p1 = p1.strip('"')
                        yield lineno, None, smart_text(p1), []



# -*- coding: utf-8 -*-

from babel import Locale, UnknownLocaleError
from django.utils.translation import get_language
try:
    from threading import local
except ImportError:
    from django.utils._threading_local import local


__all__ = ['get_current_locale', 'LocaleMiddleware']

_thread_locals = local()


def get_current_locale():
    """Get current locale data outside views.

    See http://babel.pocoo.org/docs/api/core/#babel.core.Locale for Locale
    objects documentation
    """
    return getattr(_thread_locals, 'locale', None)


class LocaleMiddleware(object):

    """Simple Django middleware that makes available a Babel `Locale` object
    via the `request.locale` attribute.
    """

    def process_request(self, request):
        try:
            code = getattr(request, 'LANGUAGE_CODE', get_language())
            locale = Locale.parse(code, sep='-')
        except (ValueError, UnknownLocaleError):
            pass
        else:
            _thread_locals.locale = request.locale = locale



# -*- coding: utf-8 -*-
import unittest

import pytest

from babel.messages import extract
from babel._compat import BytesIO

from django_babel.extract import extract_django


default_keys = extract.DEFAULT_KEYWORDS.keys()


class ExtractDjangoTestCase(unittest.TestCase):
    # TODO: translator comments are not yet supported!

    def test_extract_no_tags(self):
        buf = BytesIO(b'nothing')
        messages = list(extract_django(buf, default_keys, [], {}))
        self.assertEqual([], messages)

    def test_extract_simple_double_quotes(self):
        buf = BytesIO(b'{% trans "Bunny" %}')
        messages = list(extract_django(buf, default_keys, [], {}))
        self.assertEqual([(1, None, u'Bunny', [])], messages)

    def test_extract_simple_single_quotes(self):
        buf = BytesIO(b"{% trans 'Bunny' %}")
        messages = list(extract_django(buf, default_keys, [], {}))
        self.assertEqual([(1, None, u'Bunny', [])], messages)

    def test_extract_simple_with_context_single_quotes(self):
        buf = BytesIO(b"{% trans 'Bunny' context 'carrot' %}")
        messages = list(extract_django(buf, default_keys, [], {}))
        self.assertEqual([(1, 'pgettext',
                           [u'carrot', u'Bunny'], [])], messages)

    def test_extract_simple_with_context_double_quotes(self):
        buf = BytesIO(b"{% trans 'Bunny' context \"carrot\" %}")
        messages = list(extract_django(buf, default_keys, [], {}))
        self.assertEqual([(1, 'pgettext',
                           [u'carrot', u'Bunny'], [])], messages)

    def test_extract_simple_with_context_with_single_quotes(self):
        buf = BytesIO(b"{% trans 'Bunny' context \"'carrot'\" %}")
        messages = list(extract_django(buf, default_keys, [], {}))
        self.assertEqual([(1, 'pgettext',
                           [u'\'carrot\'', u'Bunny'], [])], messages)

    def test_extract_simple_with_context_with_double_quotes(self):
        buf = BytesIO(b"{% trans 'Bunny' context '\"carrot\"' %}")
        messages = list(extract_django(buf, default_keys, [], {}))
        self.assertEqual([(1, 'pgettext',
                           [u'"carrot"', u'Bunny'], [])], messages)

    def test_extract_var(self):
        buf = BytesIO(b'{% blocktrans %}{{ anton }}{% endblocktrans %}')
        messages = list(extract_django(buf, default_keys, [], {}))
        self.assertEqual([(1, None, u'%(anton)s', [])], messages)

    def test_extract_filter_with_filter(self):
        test_tmpl = (
            b'{% blocktrans with berta=anton|lower %}'
            b'{{ berta }}{% endblocktrans %}'
        )
        buf = BytesIO(test_tmpl)
        messages = list(extract_django(buf, default_keys, [], {}))
        self.assertEqual([(1, None, u'%(berta)s', [])], messages)

    def test_extract_with_interpolation(self):
        buf = BytesIO(b'{% blocktrans %}xxx{{ anton }}xxx{% endblocktrans %}')
        messages = list(extract_django(buf, default_keys, [], {}))
        self.assertEqual([(1, None, u'xxx%(anton)sxxx', [])], messages)

    def test_extract_unicode(self):
        buf = BytesIO(u'{% trans "@ſðæ314“ſſ¶ÐĐÞ→SÆ^ĸŁ" %}'.encode('utf8'))
        messages = list(extract_django(buf, default_keys, [], {}))
        self.assertEqual([(1, None, u'@ſðæ314“ſſ¶ÐĐÞ→SÆ^ĸŁ', [])], messages)

    def test_extract_unicode_blocktrans(self):
        test_tmpl = u'{% blocktrans %}@ſðæ314“ſſ¶ÐĐÞ→SÆ^ĸŁ{% endblocktrans %}'
        buf = BytesIO(test_tmpl.encode('utf8'))
        messages = list(extract_django(buf, default_keys, [], {}))
        self.assertEqual([(1, None, u'@ſðæ314“ſſ¶ÐĐÞ→SÆ^ĸŁ', [])], messages)

    # TODO: Yet expected to not extract the comments.
    def test_extract_ignored_comment(self):
        buf = BytesIO(
            b'{# ignored comment #1 #}{% trans "Translatable literal #9a" %}',
        )
        messages = list(extract_django(buf, default_keys, [], {}))
        self.assertEqual(
            [(1, None, u'Translatable literal #9a', [])], messages
        )

    def test_extract_ignored_comment2(self):
        test_tmpl = (
            b'{# Translators: ignored i18n comment #1 #}'
            b'{% trans "Translatable literal #9a" %}'
        )
        buf = BytesIO(test_tmpl)
        messages = list(extract_django(buf, default_keys, [], {}))
        self.assertEqual(
            [(1, None, u'Translatable literal #9a', [])], messages
        )

    def test_extract_valid_comment(self):
        test_tmpl = (
            b'{# ignored comment #6 #}'
            b'{% trans "Translatable literal #9h" %}'
            b'{# Translators: valid i18n comment #7 #}'
        )
        buf = BytesIO(test_tmpl)
        messages = list(extract_django(buf, default_keys, [], {}))
        self.assertEqual(
            [(1, None, u'Translatable literal #9h', [])], messages
        )

    def test_extract_singular_form(self):
        test_tmpl = (
            b'{% blocktrans count counter=number %}'
            b'singular{% plural %}{{ counter }} plural'
            b'{% endblocktrans %}'
        )
        buf = BytesIO(test_tmpl)
        messages = list(extract_django(buf, default_keys, [], {}))
        self.assertEqual(
            [(1, 'ngettext', (u'singular', u'%(counter)s plural'), [])],
            messages
        )

    def test_trans_blocks_must_not_include_other_block_tags(self):
        buf = BytesIO(b'{% blocktrans %}{% other_tag %}{% endblocktrans %}')
        gen = extract_django(buf, default_keys, [], {})
        with pytest.raises(SyntaxError):
            next(gen)

    def test_extract_var_other(self):
        buf = BytesIO(b'{{ book }}')
        messages = list(extract_django(buf, default_keys, [], {}))
        self.assertEqual([], messages)

    def test_extract_filters_default_translatable(self):
        buf = BytesIO(b'{{ book.author|default:_("Unknown") }}')
        messages = list(extract_django(buf, default_keys, [], {}))
        self.assertEqual([(1, None, u'Unknown', [])], messages)

    def test_extract_filters_default_translatable_single_quotes(self):
        buf = BytesIO(b"{{ book.author|default:_('Unknown') }}")
        messages = list(extract_django(buf, default_keys, [], {}))
        self.assertEqual([(1, None, u'Unknown', [])], messages)

    def test_extract_constant_single_quotes(self):
        buf = BytesIO(b"{{ _('constant') }}")
        messages = list(extract_django(buf, default_keys, [], {}))
        self.assertEqual([(1, None, u"'constant'", [])], messages)

    def test_extract_constant_double_quotes(self):
        buf = BytesIO(b'{{ _("constant") }}')
        messages = list(extract_django(buf, default_keys, [], {}))
        self.assertEqual([(1, None, u'"constant"', [])], messages)

    def test_extract_constant_block(self):
        buf = BytesIO(b'{% _("constant") %}')
        messages = list(extract_django(buf, default_keys, [], {}))
        self.assertEqual([(1, None, u'"constant"', [])], messages)

    def test_extract_constant_in_block(self):
        test_tmpl = (
            b'{% blocktrans foo=_("constant") %}{{ foo }}{% endblocktrans %}'
        )
        buf = BytesIO(test_tmpl)
        messages = list(extract_django(buf, default_keys, [], {}))
        self.assertEqual(
            [(1, None, u'"constant"', []), (1, None, u'%(foo)s', [])],
            messages,
        )

    def test_extract_context_in_block(self):
        test_tmpl = (
            b'{% blocktrans context "banana" %}{{ foo }}{% endblocktrans %}'
        )
        buf = BytesIO(test_tmpl)
        messages = list(extract_django(buf, default_keys, [], {}))
        self.assertEqual(
            [(1, 'pgettext', [u'banana', u'%(foo)s'], [])],
            messages,
        )

    def test_extract_context_in_plural_block(self):
        test_tmpl = (
            b'{% blocktrans context "banana" %}{{ foo }}'
            b'{% plural %}{{ bar }}{% endblocktrans %}'
        )
        buf = BytesIO(test_tmpl)
        messages = list(extract_django(buf, default_keys, [], {}))
        self.assertEqual(
            [(1, 'npgettext', [u'banana', u'%(foo)s', u'%(bar)s'], [])],
            messages,
        )



# -*- coding: utf-8 -*-

from babel import support as babel_support
from babel import core as babel_core
from django.conf import settings
from django.template import Library
from django.utils.translation import to_locale, get_language
try:
    from pytz import timezone
except ImportError:
    timezone = None

from django_babel.middleware import get_current_locale

__all__ = [
    'datefmt', 'datetimefmt', 'timefmt', 'numberfmt', 'decimalfmt',
    'currencyfmt', 'percentfmt', 'scientificfmt',
]

register = Library()


def _get_format():
    locale = get_current_locale()
    if not locale:
        locale = babel_core.Locale.parse(to_locale(get_language()))
    if timezone:
        tzinfo = timezone(settings.TIME_ZONE)
    else:
        tzinfo = None
    return babel_support.Format(locale, tzinfo)


def datefmt(date=None, format='medium'):
    return _get_format().date(date, format=format)
datefmt = register.filter(datefmt)


def datetimefmt(datetime=None, format='medium'):
    return _get_format().datetime(datetime, format=format)
datetimefmt = register.filter(datetimefmt)


def timefmt(time=None, format='medium'):
    return _get_format().time(time, format=format)
timefmt = register.filter(timefmt)


def numberfmt(number):
    return _get_format().number(number)
numberfmt = register.filter(numberfmt)


def decimalfmt(number, format=None):
    return _get_format().decimal(number, format=format)
decimalfmt = register.filter(decimalfmt)


def currencyfmt(number, currency):
    return _get_format().currency(number, currency)
currencyfmt = register.filter(currencyfmt)


def percentfmt(number, format=None):
    return _get_format().percent(number, format=format)
percentfmt = register.filter(percentfmt)


def scientificfmt(number):
    return _get_format().scientific(number)
scientificfmt = register.filter(scientificfmt)



# -*- coding: utf-8 -*-
import os
from distutils.dist import Distribution
from optparse import make_option
from subprocess import call

from django.core.management.base import LabelCommand, CommandError
from django.conf import settings


__all__ = ['Command']


class Command(LabelCommand):

    args = '[makemessages] [compilemessages]'

    option_list = LabelCommand.option_list + (
        make_option(
            '--locale', '-l', default=None, dest='locale', action='append',
            help=(
                'Creates or updates the message files for the given locale(s)'
                ' (e.g pt_BR). Can be used multiple times.'
            ),
        ),
        make_option(
            '--domain', '-d', default='django', dest='domain',
            help='The domain of the message files (default: "django").',
        ),
        make_option(
            '--mapping-file', '-F', default=None, dest='mapping_file',
            help='Mapping file',
        )
    )

    def handle_label(self, command, **options):
        if command not in ('makemessages', 'compilemessages'):
            raise CommandError(
                "You must either apply 'makemessages' or 'compilemessages'"
            )

        if command == 'makemessages':
            self.handle_makemessages(**options)
        if command == 'compilemessages':
            self.handle_compilemessages(**options)

    def handle_makemessages(self, **options):
        locale_paths = list(settings.LOCALE_PATHS)
        domain = options.pop('domain')
        locales = options.pop('locale')

        # support for mapping file specification via setup.cfg
        # TODO: Try to support all possible options.
        distribution = Distribution()
        distribution.parse_config_files(distribution.find_config_files())

        mapping_file = options.pop('mapping_file', None)
        has_extract = 'extract_messages' in distribution.command_options
        if mapping_file is None and has_extract:
            opts = distribution.command_options['extract_messages']
            try:
                mapping_file = opts['mapping_file'][1]
            except (IndexError, KeyError):
                mapping_file = None

        for path in locale_paths:
            potfile = os.path.join(path, '%s.pot' % domain)

            if not os.path.exists(path):
                os.makedirs(path)

            if not os.path.exists(potfile):
                with open(potfile, 'wb') as fobj:
                    fobj.write(b'')

            cmd = ['pybabel', 'extract', '-o', potfile]

            if mapping_file is not None:
                cmd.extend(['-F', mapping_file])

            cmd.append(os.path.dirname(os.path.relpath(path)))

            call(cmd)

            for locale in locales:
                pofile = os.path.join(
                    os.path.dirname(potfile),
                    locale,
                    'LC_MESSAGES',
                    '%s.po' % domain)

                if not os.path.isdir(os.path.dirname(pofile)):
                    os.makedirs(os.path.dirname(pofile))

                if not os.path.exists(pofile):
                    with open(pofile, 'wb') as fobj:
                        fobj.write(b'')

                cmd = ['pybabel', 'update', '-D', domain,
                       '-i', potfile,
                       '-d', os.path.relpath(path),
                       '-l', locale]
                call(cmd)

    def handle_compilemessages(self, **options):
        locale_paths = list(settings.LOCALE_PATHS)
        domain = options.pop('domain')
        locales = options.pop('locale')

        for path in locale_paths:
            for locale in locales:
                po_file = os.path.join(
                    path, locale, 'LC_MESSAGES', domain + '.po'
                )
                if os.path.exists(po_file):
                    cmd = ['pybabel', 'compile', '-D', domain,
                           '-d', path, '-l', locale]
                    call(cmd)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import codecs
from setuptools import setup, find_packages


def read(*parts):
    filename = os.path.join(os.path.dirname(__file__), *parts)
    with codecs.open(filename, encoding='utf-8') as fp:
        return fp.read()


setup(
    name='django-babel',
    description='Utilities for using Babel in Django',
    long_description=read('README.rst') + u'\n\n' + read('CHANGELOG.rst'),
    version='0.5.0',
    license='BSD',
    author='Christopher Grebs',
    author_email='cg@webshox.org',
    maintainer='Thomas Grainger',
    maintainer_email='django-babel@graingert.co.uk',
    url='https://github.com/python-babel/django-babel/',
    packages=find_packages(exclude=('tests',)),
    install_requires=[
        'django>=1.4,<1.10',
        'babel>=1.3',
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Framework :: Django',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Programming Language :: Python :: Implementation :: CPython',
    ],
    entry_points={
        'babel.extractors': [
            'django = django_babel.extract:extract_django',
        ]
    }
)
